'                        Trig Values V1.00

'Author:  Sid Weaver
'Date  :  27 August 2007

'This program provides the sine, cosine and tangent for all angles
'from 0 to 90 degrees.



CON
  _clkmode    = xtal1 + pll16x
  _xinfreq    = 5_000_000

  vgaPinGroup = 8                             ' Base pin for VGA pin group
  ps2KeyData  = 26                             ' Data pin for PS/2 keyboard
  ps2KeyClock = 27                             ' Clock pin for PS/2 keyboard


  
OBJ
  ldr     : "OS_loader"                        ' SPIN program loader and support routines
  ldrInit : "OS_loaderInit"                    '  Initialization and assembly portion
  text    : "OS_vgaHiresText"                  ' Hires VGA driver modified for OS use
  vgaInit : "OS_vgaHiresTextInit"              '  Initialization and assembly portion
  key     : "OS_keyboardA"                      ' PS/2 keyboard driver modified for OS use
  keyInit : "OS_keyboardInitA"                  '  Initialization and assembly portion

  
CON
  _free = (ldr#endMemory - ldr#endFree + ldr#totalNeeded + 3) / 4  ' # longs needed

VAR
  long selection, z
  word a '  byteval,
  byte  newkey,x, buffer[32]

  
PUB initialize | t, i, p
  if long[ldr#initMarker] <> ldr#uniqueMark                ' Check for a valid initialization
    repeat i from 0 to 7
      if i <> cogid                                        ' Stop all cogs except this one
        cogstop(i)
    long[ldr#memPtr]    := ldr#endFree                     ' Start at end of HUB memory
    p := ldr.allocatePRI(ldr#systemSize)                   ' Allocate system work area
    long[ldr#systemPtr] := p
    long[p+ldr#sys_cog] := |<cogid                         ' Mark current cog in use
    byte[p+ldr#sys_cogMain] := cogid + 1
    long[p+ldr#sys_random] := ldr#initMarker               ' Initialize random seed
    wordfill(p+ldr#sys_eeproms,0,ldr#maxPinPairs)          ' Clear EEPROMs bitmap table
    long[ldr#videoPtr]  := ldr.allocatePRI(ldr#videoSize)  ' Allocate work area for video
    long[ldr#screenPtr] := ldr.allocatePRI(ldr#screenSize) ' Allocate screen buffer
    long[ldr#colorPtr]  := ldr.allocatePRI(ldr#colorSize)  ' Allocate screen color buffer
    long[ldr#fontPtr]   := ldr.allocatePRI(ldr#fontSize)   ' Allocate font table
    long[ldr#cursorPtr] := ldr.allocatePRI(ldr#cursorSize) ' Allocate cursor control block
    if not vgaInit.start(vgaPinGroup)                      ' Now start up the VGA driver
      abort false
    long[ldr#keybdPtr]  := ldr.allocatePRI(ldr#keybdSize)  ' Allocate work area for keyboard
    if not keyInit.start(ps2KeyData,ps2KeyClock)           ' Now start up the keyboard driver
      abort false
    if not ldr.start(ldrInit.getEntryPoint)                ' Now start up the I2C driver
      abort false                                         
    long[ldr#userPtr] := ldr#noSuchAddr
    long[ldr#initMarker] := ldr#uniqueMark                 ' Basic initialization complete
    waitcnt(clkfreq * 2 + cnt)                             ' Wait for keyboard to initialize
    text.out(text#Cr)
  if not key.present
    text.str(string("--> keyboard not present, can't continue"))
    long[ldr#initMarker] := 0
    abort false
  start

PUB start
  waitcnt(clkfreq + cnt)                 ' Wait for simulator to initialize
  start1

PUB start1

  start2

PUB start2
  
  repeat
    text.out(0)
    showMenu
    case selection := getChoice
      "T":
        vgainit.stop
        key.getkey
        vgainit.start(vgapingroup)
      "S":
        repeat z from 0 to 39
          text.out4($13,z,240,10)
        text.out(0)
        'text.str(string("This option gives the sine, cosine or tangent of the selected degree.",13,13))
        'text.str(string("***********************************************************************",13,13))
        'text.str(string("Press any key to continue"))
        'key.getkey
        text.out(0)
        setxy(2,4)
        readop     'Line 123

PUB setxy(n,m)
  text.out(2)
  text.out(n)
  text.out(m)

PRI showMenu
  repeat z from 0 to 39
    text.out4($13,z,240,10)
  text.out(13)
  text.str(string(19,1,10,240,"                     Trig Values V1.00",13,13,13))
  text.str(string(" T - Toggle VGA display",13,13))
  text.str(string(" S - Search for trig functions by angle",13,13))
  setxy(2,16)
  text.str(string("    Select: "))

PRI getChoice : selectedOption
{ Accept menu choice. Returns uppercase letter. }
  'key.clearkeys
  repeat
    repeat until selectedOption := key.key
    if (selectedOption => "a") and (selectedOption =< "z")                         
      selectedOption -= 32    '32 = ("a" - "A")                     'Convert letters to uppercase
  until (lookdown (selectedOption : "TS"))
  
PUB readop | p
  p := 0
  text.str(string("Key in degree, followed by Enter:    "))
  repeat until  (buffer[p] :=key.getkey) == 13 
    text.out(buffer[p++])
  buffer[p] := 0
  text.out(13)
  text.str(@buffer) 
  testop(@buffer)
  text.out(13)
  

PUB testop(nameAddr) | index
  index := lookupop(nameAddr,@opNo)
  if index == -1
    text.out(13) 
    text.str(string(19,9,62,10,"Operator not found",13))
    text.str(string(19,10,62,10,"Press any key to return to Menu",13))
      if key.getkey
        start2
  text.out(0)
  repeat 4
    text.out(13)
  setxy(2,6)
  text.str(@@opDesc[index])
  setxy(2,12)
  text.str(string("Press C to continue operator search",13))
  setxy(2,14)
  text.str(string("Press R to return to Menu",13))
  if key.newkey == "c"
    text.out(0)
    readop
  else
    start2
    
PUB lookupop(nameAddr, tblAddr) : index
  index := 0
  repeat until byte[tblAddr] == 0
    if strcomp(nameAddr,tblAddr)
      return
    index++
    tblAddr += strsize(tblAddr) + 1
  return -1              
      


DAT

                
opNo  
        byte  "0",0
        byte  "1",0
        byte  "2",0
        byte  "3",0
        byte  "4",0
        byte  "5",0
        byte  "6",0
        byte  "7",0
        byte  "8",0
        byte  "9",0
        byte  "10",0
        byte  "11",0
        byte  "12",0
        byte  "13",0
        byte  "14",0     
        byte  "15",0
        byte  "16",0  
        byte  "17",0
        byte  "18",0 
        byte  "19",0  
        byte  "20",0   
        byte  "21",0   
        byte  "22",0   
        byte "23",0  
        byte "24",0   
        byte "25",0  
        byte "26",0   
        byte "27",0   
        byte "28",0   
        byte "29",0    
        byte "30",0   
        byte  "31",0   
        byte  "32",0   
        byte  "33",0   
        byte "34",0  
        byte "35",0   
        byte "36",0  
        byte "37",0   
        byte "38",0   
        byte "39",0   
        byte "40",0    
        byte "41",0 
        byte "42",0   
        byte "43",0   
        byte "44",0  
        byte "45",0   
        byte "46",0  
        byte "47",0   
        byte "48",0   
        byte "49",0   
        byte "50",0
        byte "51",0 
        byte "52",0   
        byte "53",0   
        byte "54",0  
        byte "55",0   
        byte "56",0  
        byte "57",0   
        byte "58",0   
        byte "59",0   
        byte "60",0
        byte "61",0 
        byte "62",0   
        byte "63",0   
        byte "64",0  
        byte "65",0   
        byte "66",0  
        byte "67",0   
        byte "68",0   
        byte "69",0   
        byte "70",0
        byte "71",0 
        byte "72",0   
        byte "73",0   
        byte "74",0  
        byte "75",0   
        byte "76",0  
        byte "77",0   
        byte "78",0   
        byte "79",0   
        byte "80",0
        byte "81",0 
        byte "82",0   
        byte "83",0   
        byte "84",0  
        byte "85",0   
        byte "86",0  
        byte "87",0   
        byte "88",0   
        byte "89",0   
        byte "90",0


        
opDesc     word  @op0Desc,@op1Desc,@op2Desc,@op3Desc,@op4Desc
           word  @op5Desc,@op6Desc,@op7Desc,@op8Desc,@op9Desc
           word  @op10Desc,@op11Desc,@op12Desc,@op13Desc,@op14Desc
           word  @op15Desc,@op16Desc,@op17Desc,@op18Desc,@op19Desc
           word  @op20Desc,@op21Desc,@op22Desc,@op23Desc,@op24Desc
           word  @op25Desc,@op26Desc,@op27Desc,@op28Desc,@op29Desc
           word  @op30Desc,@op31Desc,@op32Desc,@op33Desc,@op34Desc
           word  @op35Desc,@op36Desc,@op37Desc,@op38Desc,@op39Desc
           word  @op40Desc,@op41Desc,@op42Desc,@op43Desc,@op44Desc
           word  @op45Desc,@op46Desc,@op47Desc,@op48Desc,@op49Desc 
           word  @op50Desc,@op51Desc,@op52Desc,@op53Desc,@op54Desc 
           word  @op55Desc,@op56Desc,@op57Desc,@op58Desc,@op59Desc 
           word  @op60Desc,@op61Desc,@op62Desc,@op63Desc,@op64Desc  
           word  @op65Desc,@op66Desc,@op67Desc,@op68Desc,@op69Desc  
           word  @op70Desc,@op71Desc,@op72Desc,@op73Desc,@op74Desc  
           word  @op75Desc,@op76Desc,@op77Desc,@op88Desc,@op79Desc 
           word  @op80Desc,@op81Desc,@op82Desc,@op83Desc,@op84Desc  
           word  @op85Desc,@op86Desc,@op87Desc,@op88Desc,@op89Desc 
           word  @op90Desc

           
             
op0Desc    byte "Sine = 0   Cosine = 1.0000   Tangent = 0",13,0

op1Desc    byte "Sine = .01745   Cosine = .99984   Tangent = .01745",13,0
           
op2desc    byte "Sine = .03489   Cosine = .99939   Tangent = .03492",13,0 

op3desc    byte "Sine = .05233   Cosine = .99862   Tangent = .05240",13,0 

op4Desc    byte "Sine = .06975   Cosine = .99756   Tangent = .06992",13,0 

op5Desc    byte "Sine = .08715   Cosine = .99619   Tangent = .08748",13,0  

op6Desc    byte "Sine = .10452   Cosine = .99452   Tangent = .10510",13,0 

op7Desc    byte "Sine = .12186   Cosine = .99254   Tangent = .12278",13,0

op8desc    byte "Sine = .13917   Cosine = .99026   Tangent = .14054",13,0 

op9Desc    byte "Sine = .15643   Cosine = .98768    Tangent = .15838",13,0 

op10Desc   byte "Sine = .17364   Cosine = .98480   Tangent = .17632",13,0 

op11desc   byte "Sine = .19080   Cosine = .98162   Tangent = .19438",13,0 

op12Desc   byte "Sine = .20791   Cosine = .97814   Tangent = .21255",13,0 

op13Desc   byte "Sine = .22495  Cosine = .97437   Tangent = .23086",13,0 

op14Desc   byte "Sine = .24192  Cosine = .97029   Tangent = .24932",13,0 

op15Desc   byte "Sine = .25889  Cosine = .96592   Tangent = .26794",13,0 

op16Desc   byte "Sine = .27563  Cosine = .96126   Tangent = .28674",13,0

op17Desc   byte "Sine = .29237  Cosine = .95630   Tangent = .30573",13,0

op18Desc   byte "Sine = .30901  Cosine = .95105   Tangent = .32491",13,0

op19Desc   byte "Sine = .32556  Cosine = .94551   Tangent = .34432",13,0

op20Desc   byte "Sine = .34202  Cosine = .93969   Tangent = .36397",13,0

op21Desc   byte "Sine = .35836  Cosine = .93358   Tangent = .38386",13,0

op22Desc   byte "Sine = .37460  Cosine = .2718   Tangent = .40402",13,0

op23Desc   byte "Sine = .39073  Cosine = .92050   Tangent = .42447",13,0
          
op24Desc   byte "Sine = .40673  Cosine = .91354   Tangent = .44522",13,0

op25Desc   byte "Sine = .42261  Cosine = .90630   Tangent = .46630",13,0

op26Desc   byte "Sine = .43837  Cosine = .89879   Tangent = .48773",13,0

op27Desc   byte "Sine = .45399  Cosine = .89100   Tangent = .50952",13,0

op28Desc   byte "Sine = .46947  Cosine = .88294   Tangent = .53170",13,0

op29Desc   byte "Sine = .48480  Cosine = .87461   Tangent = .55430",13,0

op30Desc   byte "Sine = .50000  Cosine = .86602   Tangent = .57735",13,0

op31Desc   byte "Sine = .51503  Cosine = .85617   Tangent = .60086",13,0   

op32Desc   byte "Sine = .52991  Cosine = .84804   Tangent = .62486",13,0

op33Desc   byte "Sine = .54462  Cosine = .83867   Tangent = .64940",13,0

op34Desc   byte "Sine = .55912  Cosine = .82903   Tangent = .67450",13,0

op35Desc   byte "Sine = .57357  Cosine = .81915   Tangent = .70020",13,0   

op36Desc   byte "Sine = .58778  Cosine = .80901   Tangent = .72654",13,0

op37Desc   byte "Sine = .60181  Cosine = .79863   Tangent = .75355",13,0   

op38Desc   byte "Sine = .61566  Cosine = .78801  Tangent = .78128",13,0

op39Desc   byte "Sine = .62932  Cosine = .77714   Tangent = .80978",13,0

op40Desc   byte "Sine = .64278  Cosine = .76604   Tangent = .83909",13,0

op41Desc   byte "Sine = .65605  Cosine = .75470   Tangent = .86928",13,0  

op42Desc   byte "Sine = .66913  Cosine = .74314   Tangent = .90040",13,0 

op43Desc   byte "Sine = .68199  Cosine = .73135   Tangent = .93251",13,0 

op44Desc   byte "Sine = .69465  Cosine = .71933   Tangent = .96568",13,0

op45Desc   byte "Sine = .70710  Cosine = .70710   Tangent = 1.0000",13,0 

op46Desc   byte "Sine = .71933  Cosine = .69465   Tangent = 1.0355",13,0 

op47Desc   byte "Sine = .73135  Cosine = .68199   Tangent = 1.0723.",13,0 

op48Desc   byte "Sine = .74314  Cosine = .66913   Tangent = 1.1106",13,0 

op49Desc   byte "Sine = .75470  Cosine = .65605   Tangent = 1.1503",13,0 

op50Desc   byte "Sine = .76604  Cosine = .64228   Tangent = 1.1917",13,0 

op51Desc   byte "Sine = .77714  Cosine = .62932   Tangent = 1.2348",13,0 

op52Desc   byte "Sine = .78801  Cosine = .61566   Tangent = 1.2799",13,0 

op53Desc   byte "Sine = .79863  Cosine = .60181   Tangent = 1.3270",13,0 

op54Desc   byte "Sine = .80901  Cosine = .58778   Tangent = 1.3763",13,0 

op55Desc   byte "Sine = .81915  Cosine = .57357   Tangent = 1.4281",13,0 

op56Desc   byte "Sine = .82903  Cosine = .55912   Tangent = 1.4825",13,0 

op57Desc   byte "Sine = .83867  Cosine = .54463   Tangent = 1.5398",13,0 

op58Desc   byte "Sine = .84804  Cosine = .52991   Tangent = 1.6003",13,0 

op59Desc   byte "Sine = .85617  Cosine = .51503   Tangent = 1.6642",13,0 

op60Desc   byte "Sine = .86602  Cosine = .50000   Tangent = 1.7320",13,0 

op61Desc   byte "Sine = .87461  Cosine = .48480   Tangent = 1.8040",13,0 

op62Desc   byte "Sine = .88294  Cosine = .46947   Tangent = 1.8807",13,0 

op63Desc   byte "Sine = .89100  Cosine = .45399   Tangent = 1.9626",13,0 

op64Desc   byte "Sine = .89879  Cosine = .43837   Tangent = 2.0503",13,0 

op65Desc   byte "Sine = .90630  Cosine = .42261   Tangent = 2.1445",13,0 

op66Desc   byte "Sine = .91354  Cosine = .40673   Tangent = 2.2460",13,0 

op67Desc   byte "Sine = .92050  Cosine = .39073   Tangent = 2.3558",13,0 

op68Desc   byte "Sine = .92718  Cosine = .37460   Tangent = 2.4750",13,0 

op69Desc   byte "Sine = .93358  Cosine = .35836   Tangent = 2.6050",13,0 

op70Desc   byte "Sine = .93969  Cosine = .34202   Tangent = 2.7474",13,0

op71Desc   byte "Sine = .94551  Cosine = .32556   Tangent = 2.9042",13,0 

op72Desc   byte "Sine = .95105  Cosine = .30901   Tangent = 3.0776",13,0 

op73Desc   byte "Sine = .95630  Cosine = .29237   Tangent = 3.2708",13,0 

op74Desc   byte "Sine = .96126  Cosine = .27563   Tangent = 3.4874",13,0 

op75Desc   byte "Sine = .96592  Cosine = .25881   Tangent = 3.7320",13,0 

op76Desc   byte "Sine = .97029  Cosine = .24192   Tangent = 4.107",13,0 

op77Desc   byte "Sine = .97437  Cosine = .22495   Tangent = 4.3314",13,0 

op78Desc   byte "Sine = .97814  Cosine = .20791   Tangent = 4.7046",13,0 

op79Desc   byte "Sine = .98162  Cosine = .19080   Tangent = 5.1445",13,0 

op80Desc   byte "Sine = .98480  Cosine = .17364   Tangent = 5.6712",13,0

op81Desc   byte "Sine = .98768  Cosine = .15643   Tangent = 6.3137",13,0 

op82Desc   byte "Sine = .99026  Cosine = .13917   Tangent = 7.1153",13,0 

op83Desc   byte "Sine = .99254  Cosine = .12186   Tangent = 8.1443",13,0 

op84Desc   byte "Sine = .99452  Cosine = .10452   Tangent = 9.5143",13,0 

op85Desc   byte "Sine = .99619  Cosine = .08715   Tangent = 11.4300",13,0 

op86Desc   byte "Sine = .99756  Cosine = .06975   Tangent = 14.3006",13,0 

op87Desc   byte "Sine = .99862  Cosine = .05233   Tangent = 19.0811",13,0 

op88Desc   byte "Sine = .99939  Cosine = .03489   Tangent = 28.6362",13,0 

op89Desc   byte "Sine = .99984  Cosine = .01745   Tangent = 57.2899",13,0 

op90Desc   byte "Sine = 1.000  Cosine = 0   Tangent = Infinity ",13,0






                 
'****************************************************************************